{% if form_field is not defined %}
    {% from 'base/_formhelpers.html' import form_field %}
{% endif %}
{% if tags %}
    <div class="highlight-tags">
        <div class="tags">
            {% for t in tags %}
                <a class="tag-link"
                   href="{{ url_for(object_tag_link, tag=t) }}">{{ t }}</a>
            {% endfor %}
        </div>
    </div>
{% endif %}
