{% extends "projects/_projects_base.html" %}
{% if form_field is not defined %}
    {% from 'base/_formhelpers.html' import form_field %}
{% endif %}
{% block content %}
    <div class="container-fluid section bottom-section text-center">
        <h3 class="margin">Reset Password for {{ email }}</h3>
        <br/>
        <div class="submission-form center">
            <form id="form-create" method="POST" enctype="multipart/form-data">
                {% for field in form %}
                    {{ form_field(field) }}
                {% endfor %}
                <div class="form-group">
                    <input id="btn-reset" type="submit"
                           class="btn btn-success btn-block btn-submit"
                           value="Submit">
                </div>
            </form>
        </div>
    </div>
{% endblock %}
