{% extends "projects/_projects_base.html" %}
{% if form_field is not defined %}
    {% from 'base/_formhelpers.html' import form_field %}
{% endif %}
{% block content %}
    <div class="container-full">
        <div class="container-fluid top-section section text-center">
            <h1>Login</h1>
        </div>
        <div class="container-fluid section bottom-section text-center">
            <img class="loading-spinner" src="{{ url_for('projects_base.static',
                                         filename='gif/dark_green_loader.gif') }}"
                 hidden>
            <h3 class="margin">Credentials</h3>
            <br/>
            <div class="submission-form center">
                <form id="form-create" method="POST"
                      enctype="multipart/form-data">
                    {% for field in form %}
                        {{ form_field(field) }}
                    {% endfor %}
                    <div class="form-group">
                        <input id="btn-login" type="submit"
                               class="btn btn-success btn-submit"
                               value="Login"/>
                        <input id="btn-register" type="button"
                               class="btn btn-warning"
                               value="Request Access"/>
                    </div>
                    <div class="form-group">
                        <a class="tag-link" id="btn-reset">Forgot Password</a>
                    </div>
                </form>
            </div>
        </div>
    </div>
{% endblock %}
