{% extends "base/base.html" %}
{% block content %}
    {% if form_field is not defined %}
        {% from 'base/_formhelpers.html' import form_field %}
    {% endif %}
    <div class="container-full">
        <div class="container-fluid top-section section text-center">
            <h1>Register a {{ item_type }}</h1>
        </div>
        <div class="container-fluid section bottom-section text-center">
            <h3 class="margin">{{ item_type }} Data</h3>
            <br/>
            <div class="submission-form center">
                <form id="form-create" method="POST"
                      enctype="multipart/form-data">
                    {% for field in form %}
                        {{ form_field(field) }}
                    {% endfor %}
                    <div class="form-group">
                        <input id="btn-create" type="submit"
                               class="btn btn-success btn-block btn-submit"
                               value="Submit">
                    </div>
                </form>
            </div>
        </div>
    </div>
{% endblock %}
